/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/AlertDialog.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: AlertDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import org.opendj.share.DjBean;
import org.opendj.share.Util;
// import org.opendj.util.mail.MailPiece;
// import org.opendj.util.mail.MailManager;
// import org.opendj.util.mail.MailException;
import org.opendj.share.LineWrapper;

public class AlertDialog extends JDialog implements ActionListener {

    public static final int TITLE_ERROR   = 0;
    public static final int TITLE_DONE    = 1;
    public static final int TITLE_WARNING = 2;

    public static final String[] TITLE_TEXTS = {
        "Error", "Done!", "WARNING", null
    };

    public static final int MSG_ENCODE_PERMISSION_DENIED             = 0;
    public static final int MSG_ERROR_CREATING_ENCODER_PROCESS       = 1;
    public static final int MSG_ERROR_READING_ENCODER_STREAM         = 2;
    public static final int MSG_ERROR_CREATING_INSTALL_LOG           = 3;
    public static final int MSG_INVALID_IP_ADDRESS                   = 4;
    public static final int MSG_INVALID_PORT                         = 5;
    public static final int MSG_ERROR_CREATING_INSTALL_DIR           = 6;
    public static final int MSG_ERROR_COPYING_FILES_TO_INSTALL_DIR   = 7;
    public static final int MSG_ERROR_CREATING_STARTUP_SCRIPT        = 8;
    public static final int MSG_ERROR_CREATING_CONFIG_FILE           = 9;
    public static final int MSG_ERROR_CREATING_START_MENU_SHORTCUT   = 10;
    public static final int MSG_INSTALL_COMPLETED                    = 11;
    public static final int MSG_ERROR_WRITING_INSTALL_LOG            = 12;
    public static final int MSG_LOGIN_FAILED                         = 13;
    public static final int MSG_ERROR_LOGGING_OUT                    = 14;
    public static final int MSG_ERROR_CREATING_ENCODER_CONFIG        = 15;
    public static final int MSG_ERROR_ENCODER_TERMINATED_PREMATURELY = 16;
    public static final int MSG_INVALID_GENRE                        = 17;
    public static final int MSG_ERROR_UPDATING_SHOW_INFO             = 18;    
    public static final int MSG_ERROR_SENDING_DIAG_EMAIL             = 19;
    public static final int MSG_ERROR_GETCHANNELTIMES_FAILED         = 20;
    public static final int MSG_ERROR_GETWAITTIME_FAILED             = 21;
    public static final int MSG_ERROR_GETAUDIOINFO_FAILED            = 22;
    public static final int MSG_ERROR_GETONAIRSTATUS_FAILED          = 23;
    public static final int MSG_ERROR_GETGENRELIST_FAILED            = 24;
    public static final int MSG_ERROR_GETMOODLIST_FAILED             = 25;
    public static final int MSG_INVALID_BITRATE                      = 26;
    public static final int MSG_INTERNET_CONNECTION_TOO_SLOW         = 27;
    public static final int MSG_PLEASE_BE_HONEST_WITH_NET_CONNECTION = 28;

    public static final String[] MSG_TEXTS = {
        "Server did not grant permission to encode",
        "An error occurred trying to create the LAME encoder process",
        "An error occurred trying to read the encoder streams",
        "An error occurred trying to create the install log!",
        "The IP address you entered is not valid",
        "The port number you entered is not valid",
        "Could not create install directory",
        "Could not copy files to the install directory",
        "Could not create startup script",
        "Could not create config file",
        "Could not create Start Menu program",
        "Installation Complete!",
        "Error writing to install log!",
        "Login failed",
        "Error logging out",
        "Could not create encoder configuration file",
        "Encoder Terminated Prematurely!  Check liveice.err for details.",
        "The genre you entered is invalid",
        "Error updating show information",
        "An error occurred trying to send the diagnostic message to OpenDJ.\n\n"
        + "Make sure you're connected to the Internet."
        + "Check your log file for more details.",
        "Could not load the times for broadcast from the server",
        "Could not get the time remaining from the server",
        "Could not get the audio server information from the server",
        "Could not get the on-air status information from the server",
        "Could not get the genre information from the server",
        "Could not get the mood information from the server",
        "Your selection for the 'Internet Connection' box was invalid",
        "Your Internet connection is too slow to install the OPENdj broadcasting software.\n\n"
        + "Sorry, but if you try to broadcast, it WILL sound terrible.\n\n"
        + "Consider getting a DSL line, cable-modem, or 56k modem connection.",
        "PLEASE be honest when you select your Internet Connection.\n\n"
        + "If you lie and say you have a faster connection "
        + "than you really have, your broadcast WILL sound terrible.",
        null
    };

    public static final int MAX_MESSAGE_PIXELS = 250;

    public static final String EMAIL_TO_NAME  = "OpenDJ Diagnostics";
    public static final String EMAIL_TO_EMAIL = "diagnostics@opendj.com";
    public static final String EMAIL_SUBJECT  = "Client-Side Error";
    public static final String EMAIL_SERVER   = "mail.opendj.com";

    private Graphics       itsGraphics        = null;
    private Font           itsFont            = null;
    private FontMetrics    itsFontMetrics     = null;
    private Main           itsMain            = null;
    private DjBean         itsDjBean          = null;
    private JFrame         itsParent          = null;
    private JButton        itsOKButton        = null;
    private JButton        itsSendButton      = null;
    private boolean        itsButtonWasPushed = false;
    private Exception      itsException       = null;

    public AlertDialog ( JFrame theParent, 
                         int theTitleID, 
                         int theMessageID ) {
        super(theParent, TITLE_TEXTS[theTitleID], true);
        init(theParent, MSG_TEXTS[theMessageID], null, null);
    }

    public AlertDialog ( JFrame theParent, 
                         int theTitleID, 
                         int theMessageID, 
                         Exception theException ) {
        super(theParent, TITLE_TEXTS[theTitleID], true);
        init(theParent, MSG_TEXTS[theMessageID], null, theException);
    }

    public AlertDialog ( JFrame theParent, 
                         int theTitleID, 
                         int theMessageID, 
                         String theExtraInfo ) {
        super(theParent, TITLE_TEXTS[theTitleID], true);
        init(theParent, MSG_TEXTS[theMessageID], theExtraInfo, null);
    }

    public AlertDialog ( JFrame theParent, 
                         int theTitleID, 
                         int theMessageID, 
                         String theExtraInfo,
                         Exception theException ) {
        super(theParent, TITLE_TEXTS[theTitleID], true);
        init(theParent, MSG_TEXTS[theMessageID], theExtraInfo, theException);
    }

    // Only use this constructor for testing purposes!
    public AlertDialog ( JFrame theParent,  
                         String theMessage,
                         Exception theException ) {
        super(theParent, "TESTING", true);        
        init(theParent, theMessage, null, theException);
    }

    private void init(JFrame theParent, 
                      String theMessage, 
                      String theExtraInfo, 
                      Exception theException) {

        itsParent = theParent;
        itsException = theException;
        itsGraphics = itsParent.getGraphics();

        theMessage += interpretExtraInfo(theExtraInfo, theException);

        // Make sure we have everything required for the exception
        if ( itsException != null ) {
            // Does Main exist?
            itsMain = Main.global();
            if ( itsMain == null ) {
                itsException = null;

            } else {
                // Is there a DJ logged in?
                itsDjBean = itsMain.getDjBean();
                if ( itsDjBean == null ) {
                    itsException = null;
                }
            }
        }

        // XXX disable email capabilities for now
        itsException = null;

        itsOKButton     = new JButton("OK");
        itsOKButton.addActionListener(this);

        itsFont = itsOKButton.getFont();
        itsFont = new Font(itsFont.getName(), itsFont.getStyle(), itsFont.getSize()-2);
        itsFontMetrics = itsGraphics.getFontMetrics(itsFont);

        // Add a message describing the exception if it's present
        if ( itsException != null ) {
            theMessage += 
                "\n\nIf you're really stumped, press the 'Send to OpenDJ' button "
                + "to send a detailed description of the error to OpenDJ.";
        }

        // Break the message up into words
        java.util.List someMessageLines = LineWrapper.wrapLines( theMessage, 
                                                                 MAX_MESSAGE_PIXELS,
                                                                 itsFontMetrics,
                                                                 itsGraphics );
        int aNumLines = someMessageLines.size();
        JPanel aLabelPane = new JPanel();
        aLabelPane.setLayout(new GridLayout(aNumLines, 1));
        JLabel aLabel = null;
        for ( int i=0; i<aNumLines; i++ ) {
            aLabel = new JLabel(someMessageLines.get(i).toString().trim());
            aLabel.setFont(itsFont);
            Main.log("Added label: '" + aLabel.getText() + "'");
            aLabelPane.add(aLabel);
        }

        JPanel aMainPane = new JPanel();
        aMainPane.setLayout(new BorderLayout());
        aMainPane.add(aLabelPane, BorderLayout.CENTER);
        if ( itsException == null ) {
            aMainPane.add(itsOKButton, BorderLayout.SOUTH);
        } else {
            itsSendButton = new JButton("Send to OpenDJ");
            itsSendButton.addActionListener(this);
            JPanel aButtonPane = new JPanel();
            aButtonPane.setLayout(new FlowLayout(FlowLayout.CENTER));
            aButtonPane.add(itsOKButton);
            aButtonPane.add(itsSendButton);
            aMainPane.add(aButtonPane, BorderLayout.SOUTH);
        }
        
        getContentPane().add(aMainPane);

        pack();
        Rectangle aParentBounds = itsParent.getBounds();
        setBounds(aParentBounds.x+Main.WINDOW_OFFSET_X,
                  aParentBounds.x+Main.WINDOW_OFFSET_Y,
                  MAX_MESSAGE_PIXELS + 20, 
                  50 + (aNumLines*itsFontMetrics.getHeight()));
        itsButtonWasPushed = false;
        setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
        if ( e.getSource() == itsOKButton ) {
            Main.log("AlertDialog: OK button pushed");
            setVisible(false);
            dispose();
            itsButtonWasPushed = true;
            return;
        }
        if ( e.getSource() == itsSendButton ) {
            if ( itsException != null ) {
                try {
                    sendExceptionToOpenDJ();
                } catch ( Exception ex ) {
                    Main.log("Could not send email: " + Util.getStackTrace(ex));
                    AlertDialog a = new AlertDialog(itsParent, 
                                                    AlertDialog.TITLE_ERROR,
                                                    AlertDialog.MSG_ERROR_SENDING_DIAG_EMAIL);
                }
            }
        }
    }

    private String interpretExtraInfo ( String theInfo, Exception theException ) {

        String rstr = Util.EMPTY_STRING;
        if ( theInfo != null && theInfo.length() > 0 ) {
            rstr += theInfo;
        }
        if ( theException != null ) {
            if ( theException instanceof java.net.ConnectException &&
                 theException.toString().indexOf("refused") != -1 ) {
                rstr += "\nThe OpenDJ server refused to connect.  The OpenDJ server might "
                    + "be down at the moment, or maybe you're not connected to the "
                    + "Internet.  You might try to ping opendj.com or get to "
                    + "the www.opendj.com website to see if either is working.";
            }
        }
        return rstr;
    }

    public boolean wasButtonPushed() {
        return itsButtonWasPushed;
    }

    private void sendExceptionToOpenDJ() /* throws MailException */ {
        /*
        // Get log string
        String aFullLog = Main.getFullLog();

        // Get all the system properties
        Properties aSysProps = System.getProperties();
        String aSysPropsString = Util.getPropertiesAsString(aSysProps);

        // Get the OpenDJ properties
        Properties aMainProps = itsMain.getProps();
        String aMainPropsString = Util.getPropertiesAsString(aMainProps);

        // Build the message
        String aMessage = "The OpenDJ broadcasting software detected an error "
            + "\n\n"
            + "The error was: " + itsException.toString() + "\n"
            + Util.getStackTrace(itsException)
            + "\n\n"
            + "The System properties were:\n"
            + aSysPropsString
            + "\n\n"
            + "The OpenDJ properties were:\n"
            + aMainPropsString
            + "\n\n"
            + "The conversation with the OpenDJ server went like this:\n"
            + aFullLog
            ;
        
        MailPiece aMail = new MailPiece ( itsDjBean.getdjname(),
                                          itsDjBean.getemail(),
                                          EMAIL_TO_NAME,
                                          EMAIL_TO_EMAIL,
                                          EMAIL_SUBJECT,
                                          aMessage );
        aMail.host = EMAIL_SERVER;
        MailManager.global().send( aMail );
        */
    }
}
