/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/ExitMonitor.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: ExitMonitor.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import org.opendj.share.Util;

public class ExitMonitor implements Runnable {

    public static final long SLEEP_INTERVAL = 100;

    private Main             itsApp            = null;
    private ChannelUpdater   itsChannelUpdater = null;
    private Server           itsServer         = null;
    private Encoder          itsEncoder        = null;
    private volatile boolean isDone            = false;

    public ExitMonitor( Main theApp, 
                        ChannelUpdater theChannelUpdater, 
                        Encoder theEncoder ) {
        itsApp = theApp;
        itsChannelUpdater = theChannelUpdater;
        itsServer = theApp.getServer();
        itsEncoder = theEncoder;
        isDone = false;
    }

    public void run () {
        
        long aSleepInterval = -1;
        while ( !isDone ) {
            try {
                aSleepInterval = itsServer.getWaitTime();
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(itsApp,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETWAITTIME_FAILED,
                                                se.getMessage());
                aSleepInterval = 0;
            } catch ( Exception e ) {
                AlertDialog a = new AlertDialog(itsApp,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETWAITTIME_FAILED,
                                                e);
                aSleepInterval = 0;
            }
            if ( aSleepInterval == 0 ) {
                killEncoder();
                return;
            }
            Util.sleep(aSleepInterval);
        }
        killEncoder();
    }

    private void killEncoder() {
        if ( itsEncoder != null ) {
            itsEncoder.finish();
            itsEncoder = null;
        }
        itsChannelUpdater.exitMonitorDone();
        itsApp.setBroadcasting(false);
    }

    public void finish() {
        isDone = true;
    }

}
