/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/LoginDialog.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: LoginDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.opendj.share.DjBean;

public class LoginDialog extends JDialog implements ActionListener {

    private Main itsApp = null;
    private JButton        itsLoginButton     = null;
    private JTextField     itsEmailTextBox    = null;
    private JPasswordField itsPasswordTextBox = null;

    public LoginDialog ( Main theMain ) {
        super(theMain, "Login", true);
        itsApp = theMain;
        init();
        pack();
        Rectangle itsAppBounds = itsApp.getBounds();
        setBounds( itsAppBounds.x+Main.WINDOW_OFFSET_X, 
                   itsAppBounds.y+Main.WINDOW_OFFSET_Y, 
                   380, 100 );
        setVisible(true);
    }

    private void init() {
        itsLoginButton     = new JButton("Login");
        itsLoginButton.addActionListener(this);
        itsEmailTextBox    = new JTextField();
        itsPasswordTextBox = new JPasswordField();

        JPanel aMainPane = new JPanel();
        aMainPane.setLayout(new GridLayout(3,2));
        aMainPane.add(new JLabel("Email:"));
        aMainPane.add(itsEmailTextBox);
        aMainPane.add(new JLabel("Password:"));
        aMainPane.add(itsPasswordTextBox);
        aMainPane.add(itsLoginButton);
        
        getContentPane().add(aMainPane);
    }

    public void actionPerformed(ActionEvent e) {

        if ( e.getSource() == itsLoginButton ) {
            Server aServer = itsApp.getServer();
            Main.log("LoginDialog: login button pushed");

            // Ask server to authenticate
            DjBean aDjBean = null;
            try {
                aDjBean = aServer.authenticate(itsEmailTextBox.getText(),
                                               itsPasswordTextBox.getText());
            } catch ( ServerException se ) {
                if ( itsApp != null ) {
                    itsApp.log(se.toString());
                }
                AlertDialog a = new AlertDialog(itsApp,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_LOGIN_FAILED,
                                                se.getMessage());
                return;

            } catch ( Exception ex ) {
                if ( itsApp != null ) {
                    itsApp.log(ex.toString());
                }
                AlertDialog a = new AlertDialog(itsApp,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_LOGIN_FAILED,
                                                ex);
                return;
            }

            if ( aDjBean != null ) {
                // yea!
                Main.log("LoginDialog: login OK!");
                itsApp.setLoggedIn(true);
                itsApp.setDjBean(aDjBean);
                setVisible(false);
                dispose();

            } else {
                Main.log("LoginDialog: login failed.");
                AlertDialog a = new AlertDialog(itsApp,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_LOGIN_FAILED);
            }
        }
    }
}
