/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/Main.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: Main.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;
import java.util.Properties;
import org.opendj.share.DjBean;
import org.opendj.share.GenreBean;
import org.opendj.share.MoodBean;
import org.opendj.share.ChannelTime;
import org.opendj.share.Util;

public class Main extends JFrame implements ActionListener, WindowListener {

    public static final String PROP_PROPFILE         = "opendj.props";
    public static final String PROP_HOST             = "server.host";
    public static final String PROP_PORT             = "server.port";
    public static final String PROP_ENCODER          = "encoder.class";
    public static final String PROP_ICECAST_HOST     = "icecast.host";
    public static final String PROP_ICECAST_PORT     = "icecast.port";    
    public static final String PROP_ICECAST_PASSWORD = "icecast.password";
    public static final String PROP_USE_BITRATE      = "icecast.bitrate.use";
    public static final String PROP_MAX_BITRATE      = "icecast.bitrate.max";

    public static final int WINDOW_OFFSET_X = 25;
    public static final int WINDOW_OFFSET_Y = 25;

    public static final String STATUS_PREFIX       = "Status: ";
    public static final String STATUS_NOTLOGGEDIN  = "Not Connected";
    public static final String STATUS_LOGGEDIN     = "Connected";
    public static final String STATUS_BROADCASTING = "BROADCASTING";

    public static final String TEXT_BUTTON_LOGIN    = "Log In";
    public static final String TEXT_BUTTON_LOGOUT   = "Log Out";
    public static final String TEXT_BUTTON_SHOWDJ   = "Your DJ Stats";
    public static final String TEXT_BUTTON_UPDATE   = "Update Info For This Broadcast";
    public static final String TEXT_BUTTON_CHAT     = "Chat";

    private Font    itsButtonFont     = null;
    private JButton itsLoginButton    = null;
    private JButton itsShowDjButton   = null;
    private JButton itsChatButton     = null;
    private JButton itsUpdateButton   = null;
    private JLabel  itsStatusLabel    = null;
    private JLabel  itsListenerLabel  = null;
    private JTable  itsChannelTable   = null;

    private java.util.List itsGenreList = null;
    private java.util.List itsMoodList  = null;
    private UpdateShowDialog itsUpdateDialog = null;
    private ChatDialog itsChatDialog = null;
    
    private String  itsHost           = null;
    private int     itsPort           = -1;
    private Server  itsServer         = null;
    private String  itsEncoderClass   = null;

    private volatile boolean itsLoggedIn       = false;
    private volatile boolean itsIsBroadcasting = false;

    private Properties     itsProps  = null;
    private DjBean         itsDjBean = null;
    private ChannelUpdater itsChannelUpdater = null;
    private OnAirStatus    itsOnAirStatus = null;

    private static Main    GLOBAL_MAIN = null;
    private static Vector  LOG_DATA    = null;
    private static final int MAX_LOG_SIZE = 500;
    private static final int LOG_KILL_SIZE = 100;

    public Main () {
        super("OPENdj");
        itsLoggedIn = false;
        itsIsBroadcasting = false;
        itsChannelUpdater = new ChannelUpdater();
        GLOBAL_MAIN = this;
    }

    public static void main ( String[] args ) {
        log("in go");
        if ( !init() ) {
            log ("init failed");
            return;
        }
        log("init succeeded");
    }

    private static boolean init () {
        
        // Init basic UI
        try {
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        } catch ( Exception e ) {
            log("Error initializing UI: " + e.toString());
            e.printStackTrace();
            return false;
        }

        // Init window components
        Main anApp = new Main();
        if ( !anApp.loadProps() ) {
            return false;
        }

        Component contents = anApp.createComponents();
        anApp.getContentPane().add(contents, BorderLayout.CENTER);

        anApp.addWindowListener(anApp);
        anApp.pack();
        anApp.setBounds( new Rectangle(50,50,500,200) );
        anApp.setVisible(true);

        // Init server
        anApp.initServer();

        return true;
    }

    private Component createComponents () {

        itsLoginButton = new JButton(TEXT_BUTTON_LOGIN);
        itsLoginButton.addActionListener(this);

        // Setup button font
        itsButtonFont = itsLoginButton.getFont();
        itsButtonFont = new Font(itsButtonFont.getName(), 
                                 itsButtonFont.getStyle(),
                                 itsButtonFont.getSize() - 4);
        itsLoginButton.setFont(itsButtonFont);

        itsShowDjButton = new JButton(TEXT_BUTTON_SHOWDJ);
        itsShowDjButton.setEnabled(false);
        itsShowDjButton.addActionListener(this);
        itsShowDjButton.setFont(itsButtonFont);

        itsUpdateButton = new JButton(TEXT_BUTTON_UPDATE);
        itsUpdateButton.setEnabled(false);
        itsUpdateButton.addActionListener(this);
        itsUpdateButton.setFont(itsButtonFont);

        itsChatButton = new JButton(TEXT_BUTTON_CHAT);
        itsChatButton.setEnabled(false);
        itsChatButton.addActionListener(this);
        itsChatButton.setFont(itsButtonFont);

        itsStatusLabel = new JLabel(STATUS_PREFIX + STATUS_NOTLOGGEDIN);
        itsListenerLabel = new JLabel(Util.EMPTY_STRING);
        
        JToolBar aToolbar = new JToolBar();
        aToolbar.setFloatable(false);
        aToolbar.setLayout(new BorderLayout());
        aToolbar.add(itsLoginButton, BorderLayout.WEST);
        JPanel anEastPanel = new JPanel();
        anEastPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        anEastPanel.add(itsStatusLabel);
        aToolbar.add(anEastPanel, BorderLayout.EAST);

        JPanel aListenerPanel = new JPanel();
        aListenerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        aListenerPanel.add(itsListenerLabel);

        JPanel aButtonPanel = new JPanel();
        aButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        aButtonPanel.add(itsShowDjButton);
        aButtonPanel.add(itsUpdateButton);
        aButtonPanel.add(itsChatButton);

        JPanel aBottomPanel = new JPanel();
        aBottomPanel.setLayout(new BorderLayout());
        aBottomPanel.add(aButtonPanel, BorderLayout.SOUTH);
        aBottomPanel.add(aListenerPanel, BorderLayout.WEST);

        String[] someHeaders = { "Channel", "Next Broadcast" };
        DefaultTableModel dm = new DefaultTableModel(someHeaders, 0);
        itsChannelTable = new JTable(dm);
        JScrollPane aSP = new JScrollPane(itsChannelTable);

        JPanel aMainPane = new JPanel();
        aMainPane.setLayout(new BorderLayout());
        aMainPane.add(aToolbar, BorderLayout.NORTH);
        aMainPane.add(aSP, BorderLayout.CENTER);
        aMainPane.add(aBottomPanel, BorderLayout.SOUTH);

        return aMainPane;
    }

    private void initServer () {
        itsServer = new Server(itsHost, itsPort);
    }

    public static void log ( String s ) {
        String aLogString = (new Date()).toString() + " [MAIN] " + s;
        if ( LOG_DATA == null ) {
            LOG_DATA = new Vector();
        }
        synchronized (LOG_DATA) {
            int aLogSize = LOG_DATA.size();
            if ( aLogSize > MAX_LOG_SIZE ) {
                for ( int i=0; i<LOG_KILL_SIZE; i++ ) {
                    LOG_DATA.remove(0);
                }
            }
            LOG_DATA.add(aLogString);
        }
        System.err.println(aLogString);
    }

    public static String getFullLog() {
        String rstr = Util.EMPTY_STRING;
        synchronized (LOG_DATA) {
            int aLogSize = LOG_DATA.size();
            for ( int i=0; i<aLogSize; i++ ) {
                rstr += LOG_DATA.get(i).toString() + "\n";
            }
        }
        return rstr;
    }
    
    public void actionPerformed(ActionEvent e) {
        if ( e.getSource() == itsLoginButton ) {
            log("LoginButton pushed, itsLoggedIn=" + itsLoggedIn);
            if ( !itsLoggedIn ) {
                // The login dialog will make a callback to us with "setLoggedIn"
                // if it is successful
                LoginDialog aLoginDialog = new LoginDialog(this);

            } else {
                if ( itsIsBroadcasting ) {
                    YesNoDialog aYesNoDialog
                        = new YesNoDialog(this,
                                          YesNoDialog.TITLE_CONFIRMLOGOUT,
                                          YesNoDialog.MSG_CONFIRMLOGOUT);
                    if ( aYesNoDialog.getButtonClicked() != YesNoDialog.YES ) {
                        return;
                    }
                }                 
                itsChannelUpdater.finish();
                try {
                    itsServer.logout();
                    setLoggedIn(false);
                    log("setLoggedIn(false) called!");
                } catch ( ServerException se ) {
                    AlertDialog a = new AlertDialog(this, 
                                                    AlertDialog.TITLE_ERROR,
                                                    AlertDialog.MSG_ERROR_LOGGING_OUT,
                                                    se.getMessage());
                } catch ( Exception ex ) {
                    AlertDialog a = new AlertDialog(this, 
                                                    AlertDialog.TITLE_ERROR,
                                                    AlertDialog.MSG_ERROR_LOGGING_OUT,
                                                    ex);
                }
            }

        } else if ( e.getSource() == itsShowDjButton ) {
            if ( itsShowDjButton.isEnabled() ) {
                ShowDjDialog s = new ShowDjDialog(this, itsDjBean);
            }

        } else if ( e.getSource() == itsChatButton ) {
            if ( itsChatButton.isEnabled() ) {
                showChatDialog();
            }

        } else if ( e.getSource() == itsUpdateButton ) {
            if ( itsUpdateButton.isEnabled() ) {
                if ( itsUpdateDialog == null ){
                    itsUpdateDialog = new UpdateShowDialog(this, this, itsDjBean.getgenre());
                } else {
                    itsUpdateDialog.setVisible(true);
                }
            }
        }
    }

    public Server getServer () {
        return itsServer;
    }

    public void setLoggedIn ( boolean theIsLoggedIn ) {
        if ( theIsLoggedIn ) {
            // We're logged in

            // Grab the audio server info
            try {
                itsServer.loadAudioServerInfo(this);
                itsLoggedIn = theIsLoggedIn;
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(this,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETAUDIOINFO_FAILED,
                                                se.getMessage());
                return;
                
            } catch ( Exception ex ) {
                AlertDialog a = new AlertDialog(this,
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETAUDIOINFO_FAILED,
                                                ex);
                return;
            }

            // Update the channel table
            itsChannelUpdater.init( this, (DefaultTableModel) itsChannelTable.getModel() );
            Thread anUpdateThread = new Thread(itsChannelUpdater);
            anUpdateThread.start();

            itsStatusLabel.setText(STATUS_PREFIX + STATUS_LOGGEDIN);
            itsLoginButton.setText(TEXT_BUTTON_LOGOUT);
            itsShowDjButton.setEnabled(true);

        } else {
            itsStatusLabel.setText(STATUS_PREFIX + STATUS_NOTLOGGEDIN);
            itsLoginButton.setText(TEXT_BUTTON_LOGIN);
            itsShowDjButton.setEnabled(false);
            itsLoggedIn = false;
        }
    }

    public void setBroadcasting ( boolean theIsBroadcasting ) {
        itsIsBroadcasting = theIsBroadcasting;
        if ( itsIsBroadcasting ) {
            itsStatusLabel.setText(STATUS_PREFIX + STATUS_BROADCASTING);
            refreshShowStats();
            itsUpdateButton.setEnabled(true);
            itsChatButton.setEnabled(true);
            showChatDialog();
        } else {
            itsStatusLabel.setText(STATUS_PREFIX + STATUS_LOGGEDIN);
            itsUpdateButton.setEnabled(false);
            // itsChatButton.setEnabled(false);
        }
    }

    public boolean getBroadcasting() {
        return itsIsBroadcasting;
    }

    public void showChatDialog () {

        try {
            if ( itsChatDialog.getRoomName() != getActiveChannel().name ) {
                itsChatDialog.finish();
                itsChatDialog = null;
            }
        } catch ( Exception e ) {
            if ( itsChatDialog != null ) {
                itsChatDialog.finish();
                itsChatDialog = null;
            }
        }
        if ( itsChatDialog == null ) {
            itsChatDialog = new ChatDialog(this, this, itsDjBean);

        } else if ( !itsChatDialog.isShowing() ) {
            itsChatDialog.setVisible(true);
        }

    }

    public void setDjBean( DjBean theDjBean ) {
        itsDjBean = theDjBean;
    }

    public DjBean getDjBean() {
        return itsDjBean;
    }

    public void refreshShowStats() {
        if ( itsIsBroadcasting ) {
            try {
                itsOnAirStatus = itsServer.getOnAirStatus();
                itsListenerLabel.setText("Current Listeners: " + itsOnAirStatus.current + "  "
                                         + "Average: " + itsOnAirStatus.avg);
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETONAIRSTATUS_FAILED,
                                                se.getMessage());
            } catch ( Exception ex ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETONAIRSTATUS_FAILED,
                                                ex);
            }
        } else {
            itsListenerLabel.setText(Util.EMPTY_STRING);
        }
    }

    public OnAirStatus getOnAirStatus() {
        return itsOnAirStatus;
    }
    
    private boolean loadProps () {
        // Init properties
        try {
            String aPropFile = System.getProperty(PROP_PROPFILE);
            itsProps = new Properties();
            itsProps.load(new FileInputStream(aPropFile));
            itsHost = itsProps.getProperty(PROP_HOST);
            itsPort = Integer.parseInt(itsProps.getProperty(PROP_PORT));
            itsEncoderClass = itsProps.getProperty(PROP_ENCODER);

            // Append 000 to the USE_BITRATE property
            itsProps.setProperty(PROP_USE_BITRATE, 
                                 itsProps.getProperty(PROP_USE_BITRATE) + "000");

        } catch ( Exception e ) {
            log("Error reading properties: " + e.toString());
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public Properties getProps() {
        return itsProps;
    }

    public Font getButtonFont() {
        return itsButtonFont;
    }

    public java.util.List getGenreList() {
        if ( itsGenreList == null ) {
            try {
                itsGenreList = itsServer.getGenreList();
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETGENRELIST_FAILED,
                                                se.getMessage());
            } catch ( Exception ex ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETGENRELIST_FAILED,
                                                ex);
            }
        }
        return itsGenreList;
    }

    public java.util.List getMoodList() {
        if ( itsMoodList == null ) {
            try {
                itsMoodList = itsServer.getMoodList();
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETMOODLIST_FAILED,
                                                se.getMessage());
            } catch ( Exception ex ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_GETMOODLIST_FAILED,
                                                ex);
            }
        }
        return itsMoodList;
    }

    public int getGenreID ( int theGenreIndex ) {
        return ((GenreBean) itsGenreList.get(theGenreIndex)).getgid();
    }

    public int getMoodID ( int theMoodIndex ) {
        return ((MoodBean) itsMoodList.get(theMoodIndex)).getmid();
    }

    public ChannelTime getActiveChannel () {
        return itsChannelUpdater.getActiveChannel();
    }

    public void setProperty( String thePropName, String thePropValue ) {
        itsProps.setProperty(thePropName, thePropValue);
    }

    public void windowClosing(WindowEvent e) {
        try { itsServer.logout(); } catch ( Exception ex ) {}
        if ( itsChatDialog != null ) { itsChatDialog.finish(); }
        System.exit(0);
    }
    public void windowOpened(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}

    public static Main global() { return GLOBAL_MAIN; }

    public void setMaxBitrate(int aMaxBitrate) {
        itsProps.setProperty(PROP_MAX_BITRATE, String.valueOf(aMaxBitrate));
        try {
            int aUseBitrate = Integer.parseInt(itsProps.getProperty(PROP_USE_BITRATE));
            if ( aUseBitrate > aMaxBitrate ) {
                itsProps.setProperty(PROP_USE_BITRATE, String.valueOf(aMaxBitrate));
            }
        } catch ( Exception e ) {
            log("Error in Main.setMatBitrate: " + e.toString());
            e.printStackTrace();
            itsProps.setProperty(PROP_USE_BITRATE, "24000");
        }
    }
}
