/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/UpdateShowDialog.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: UpdateShowDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Iterator;
import org.opendj.share.DjBean;
import org.opendj.share.GenreBean;
import org.opendj.share.MoodBean;
import org.opendj.share.Util;

public class UpdateShowDialog extends JDialog implements ActionListener {

    private JComboBox      itsGenreComboBox    = null;
    private JComboBox      itsMoodComboBox     = null;
    private JTextArea      itsCommentsTextArea = null;
    private JButton        itsUpdateButton     = null;
    private JButton        itsCancelButton     = null;
    private Main           itsApp              = null;
    private Font           itsFont             = null;
    private int            itsGenreIndex       = -1;
    private int            itsMoodIndex        = -1;
    private String         itsComments         = null;

    public UpdateShowDialog ( JFrame theParent, Main theApp, int theInitialGenre ) {
        super(theParent, "Show Meta Data", true);
        itsApp = theApp;
        itsFont = theApp.getButtonFont();
        init(theInitialGenre);
        pack();
        Rectangle aParentBounds = theParent.getBounds();
        setBounds(aParentBounds.x+Main.WINDOW_OFFSET_X,
                  aParentBounds.x+Main.WINDOW_OFFSET_Y,
                  400, 200);
        setVisible(true);
    }

    private void init ( int theInitialGenre ) {

        itsUpdateButton     = new JButton("Update");
        itsUpdateButton.addActionListener(this);
        itsCancelButton     = new JButton("Cancel");
        itsCancelButton.addActionListener(this);

        itsGenreComboBox = new JComboBox();
        itsMoodComboBox = new JComboBox();
        itsCommentsTextArea = new JTextArea();
        JScrollPane aCommentsScrollPane = new JScrollPane(itsCommentsTextArea);

        JPanel aComboBoxPane = new JPanel();
        aComboBoxPane.setLayout(new GridLayout(2,2));

        JLabel aLabel = new JLabel("Genre for this show:");
        aLabel.setFont(itsFont);
        aComboBoxPane.add( aLabel );
        aComboBoxPane.add( itsGenreComboBox );

        aLabel = new JLabel("Mood for this show:");
        aLabel.setFont(itsFont);
        aComboBoxPane.add( aLabel );
        aComboBoxPane.add( itsMoodComboBox );

        JPanel aCommentsPane = new JPanel();
        aCommentsPane.setLayout(new BorderLayout());
        
        aLabel = new JLabel("Your comments:");
        aLabel.setFont(itsFont);
        aCommentsPane.add( aLabel, BorderLayout.WEST );
        aCommentsPane.add( aCommentsScrollPane, BorderLayout.CENTER );

        JPanel aButtonPane = new JPanel();
        aButtonPane.setLayout(new BorderLayout());
        aButtonPane.add(new JLabel(" "), BorderLayout.NORTH); // just for spacing
        aButtonPane.add(itsUpdateButton, BorderLayout.WEST);
        aButtonPane.add(itsCancelButton, BorderLayout.EAST);

        JPanel aMainPane = new JPanel();
        aMainPane.setLayout(new BorderLayout());
        aMainPane.add(aComboBoxPane, BorderLayout.NORTH);
        aMainPane.add(aCommentsPane, BorderLayout.CENTER);
        aMainPane.add(aButtonPane, BorderLayout.SOUTH);

        populateFields(theInitialGenre);
        
        getContentPane().add(aMainPane);
    }

    public void actionPerformed(ActionEvent e) {
        if ( e.getSource() == itsCancelButton ) {
            Main.log("UpdateShowDialog: cancel button pushed");
            itsGenreComboBox.setSelectedIndex(itsGenreIndex);
            itsMoodComboBox.setSelectedIndex(itsMoodIndex);
            itsCommentsTextArea.setText(itsComments);
            setVisible(false);

        } else if ( e.getSource() == itsUpdateButton ) {
            Main.log("UpdateShowDialog: update button pushed");

            // Extract the values
            itsGenreIndex = itsGenreComboBox.getSelectedIndex();
            int aGenreID = itsApp.getGenreID(itsGenreIndex);
            if ( aGenreID == -1 ) {
                AlertDialog a = new AlertDialog(itsApp, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_INVALID_GENRE);
                return;
            }

            itsMoodIndex = itsMoodComboBox.getSelectedIndex();
            int aMoodID = 1;
            if ( itsMoodIndex > 0 ) {
                // Only change aMoodID if it was not "unspecified"
                // because "unspecified" does not exist in the App's MoodList
                aMoodID = itsApp.getMoodID(itsMoodIndex-1);
            }

            itsComments = itsCommentsTextArea.getText();

            Server aServer = itsApp.getServer();
            try {
                aServer.updateShowMetaData( aGenreID, 
                                            aMoodID, 
                                            itsComments );
            } catch ( ServerException se ) {
                AlertDialog a = new AlertDialog(itsApp, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_UPDATING_SHOW_INFO,
                                                se.getMessage());
            } catch ( Exception ex ) {
                AlertDialog a = new AlertDialog(itsApp, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_UPDATING_SHOW_INFO,
                                                ex);
            }

            setVisible(false);
        }
    }

    private void populateFields( int theInitialGenre ) {

        java.util.List aGenreList = itsApp.getGenreList();
        java.util.List aMoodList = itsApp.getMoodList();

        Iterator i = aGenreList.iterator();
        GenreBean aGenreBean = null;
        int idx=0;
        while ( i.hasNext() ) {
            aGenreBean = (GenreBean) i.next();
            itsGenreComboBox.addItem( aGenreBean.getname() );
            if ( aGenreBean.getgid() == theInitialGenre ) {
                itsGenreIndex = idx;
            }
            idx++;
        }
        itsGenreComboBox.setSelectedIndex(itsGenreIndex);

        i = aMoodList.iterator();
        MoodBean aMoodBean = null;
        itsMoodComboBox.addItem( "not specified" );
        while ( i.hasNext() ) {
            aMoodBean = (MoodBean) i.next();
            itsMoodComboBox.addItem( aMoodBean.getname() );
        }
        itsMoodIndex = 0;
        itsMoodComboBox.setSelectedIndex(itsMoodIndex);

        itsComments = Util.EMPTY_STRING;
    }
}
