/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/YesNoDialog.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: YesNoDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import org.opendj.share.DjBean;
import org.opendj.share.Util;
import org.opendj.share.LineWrapper;

public class YesNoDialog extends JDialog implements ActionListener {

    public static final int TITLE_CONFIRMLOGOUT = 0;

    public static final String[] TITLE_TEXTS = {
        "Confirm Log Out", null
    };

    public static final int MSG_CONFIRMLOGOUT             = 0;

    public static final String[] MSG_TEXTS = {
        "Are you sure you want to log out?\nYour current broadcast will be stopped!",
        null
    };

    public static final int MAX_MESSAGE_PIXELS = 250;
    public static final int NOT_PUSHED = -1;
    public static final int NO         = 0;
    public static final int YES        = 1;

    private Graphics       itsGraphics        = null;
    private Font           itsFont            = null;
    private FontMetrics    itsFontMetrics     = null;
    private Main           itsMain            = null;
    private JFrame         itsParent          = null;
    private JButton        itsOKButton        = null;
    private JButton        itsCancelButton    = null;
    private int            itsPushedButton    = NOT_PUSHED;

    public YesNoDialog ( JFrame theParent, 
                         int theTitleID, 
                         int theMessageID ) {
        super(theParent, TITLE_TEXTS[theTitleID], true);
        init(theParent, MSG_TEXTS[theMessageID]);
    }

    private void init(JFrame theParent, 
                      String theMessage) {

        itsParent = theParent;
        itsGraphics = itsParent.getGraphics();

        itsOKButton     = new JButton("OK");
        itsOKButton.addActionListener(this);
        itsCancelButton = new JButton("Cancel");
        itsCancelButton.addActionListener(this);

        itsFont = itsOKButton.getFont();
        itsFont = new Font(itsFont.getName(), itsFont.getStyle(), itsFont.getSize()-2);
        itsFontMetrics = itsGraphics.getFontMetrics(itsFont);

        // Break the message up into words
        java.util.List someMessageLines = LineWrapper.wrapLines( theMessage, 
                                                                 MAX_MESSAGE_PIXELS,
                                                                 itsFontMetrics,
                                                                 itsGraphics );
        int aNumLines = someMessageLines.size();
        JPanel aLabelPane = new JPanel();
        aLabelPane.setLayout(new GridLayout(aNumLines, 1));
        JLabel aLabel = null;
        for ( int i=0; i<aNumLines; i++ ) {
            aLabel = new JLabel(someMessageLines.get(i).toString().trim());
            aLabel.setFont(itsFont);
            Main.log("Added label: '" + aLabel.getText() + "'");
            aLabelPane.add(aLabel);
        }

        JPanel aMainPane = new JPanel();
        aMainPane.setLayout(new BorderLayout());
        aMainPane.add(aLabelPane, BorderLayout.CENTER);

        JPanel aButtonPane = new JPanel();
        aButtonPane.setLayout(new BorderLayout());
        aButtonPane.add(itsOKButton,     BorderLayout.WEST);
        aButtonPane.add(itsCancelButton, BorderLayout.EAST);
        aMainPane.add(aButtonPane, BorderLayout.SOUTH);
        
        getContentPane().add(aMainPane);

        pack();
        Rectangle aParentBounds = itsParent.getBounds();
        setBounds(aParentBounds.x+Main.WINDOW_OFFSET_X,
                  aParentBounds.x+Main.WINDOW_OFFSET_Y,
                  MAX_MESSAGE_PIXELS + 20, 
                  50 + (aNumLines*itsFontMetrics.getHeight()));
        itsPushedButton = NOT_PUSHED;
        setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
        if ( e.getSource() == itsOKButton ) {
            Main.log("YesNoDialog: OK button pushed");
            setVisible(false);
            dispose();
            itsPushedButton = YES;
        }
        if ( e.getSource() == itsCancelButton ) {
            Main.log("YesNoDialog: OK button pushed");
            setVisible(false);
            dispose();
            itsPushedButton = NO;
        }
    }

    public int getButtonClicked() { return itsPushedButton; }
}
