/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/java/install/Installer.java,v 1.2 2002/05/06 09:21:08 jonathan Exp $
 * $Log: Installer.java,v $
 * Revision 1.2  2002/05/06 09:21:08  jonathan
 * Updated for new IP address of opendj.com.  At some point this should be brought out into a config file instead of being hard-coded, but for now it's nice that it's only in this one place.
 *
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client.java.install;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.sampled.*;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Vector;
import org.opendj.client.AlertDialog;
import org.opendj.share.Util;

public class Installer extends JFrame implements ActionListener, WindowListener {

    public static final String PROP_JAVA        = "path.to.java";
    public static final String PROP_RT          = "path.to.rt";
    public static final String PROP_LAME        = "path.to.lame";
    public static final String PROP_INSTALL_DIR = "path.to.install";
    public static final String PROP_SETUP_DIR   = "path.to.setup";

    public static final String START_COMMAND_NAME = "OPENdj.bat";
    public static final String CONFIG_FILE_NAME   = "opendj.properties";

    public static final String SERVER_IP   = "66.92.188.106";
    public static final String SERVER_PORT = "8025";

    public static final String TEXT_INSTALL_BUTTON = "Install";
    public static final String TEXT_CANCEL_BUTTON  = "Cancel";

    private PrintWriter itsDebugWriter = null;

    private Vector     itsAudioFormatVec   = null;
    private JComboBox  itsAudioFormatCombo = null;
    private JComboBox  itsBitrateCombo     = null;
    private JTextField itsServerIP         = null;
    private JTextField itsServerPort       = null;
    private JButton    itsInstallButton    = null;
    private JButton    itsCancelButton     = null;

    private AudioFormat           itsAudioFormat    = null;
    private TargetDataLine        itsInputLine      = null;

    private String SETUP_DIR = null;

    public static final String[] ENCODING_NAMES 
        = { "PCM_SIGNED", "PCM_UNSIGNED", "ALAW", "ULAW" };
    public static final AudioFormat.Encoding[] ENCODING_VALUES
        = { AudioFormat.Encoding.PCM_SIGNED,
            AudioFormat.Encoding.PCM_UNSIGNED,
            AudioFormat.Encoding.ALAW,
            AudioFormat.Encoding.ULAW };
    public static final float[]  FREQUENCIES     = { 44100f, 22050f, 16000f, 11025f, 8000f };
    public static final String[] FREQUENCY_NAMES = { "44.1 kHz", "22.05 kHz", "16 kHz", "11.025 kHz", "8 kHz"};
    public static final int[]    SAMPLE_BITS     = { 16, 8 };
    public static final int[]    NUM_CHANNELS    = { 2, 1 };
    public static final String[] CHANNEL_NAMES   = { "stereo", "mono" };
    public static final boolean  ISBIGENDIAN     = true;
    public static final boolean  PCM_SIGNED      = true;
    public static final int[] BITRATES = { -1, -1, 24, 24, 24 };
    public static final String[] BITRATES_DESC = { "None", 
                                                   "Slower than 56k Modem", 
                                                   "56k Modem", 
                                                   "Cable Modem or DSL", 
                                                   "T1 or faster" };


    public static void main ( String[] argv ) {
        Installer i = new Installer();
        try {
            i.go();
        } catch ( Exception e ) {
            i.log("Exception in Installer: " + e.toString());
            e.printStackTrace();
        }
	}

    public Installer () {
        super("OPENdj Installer");
    }

    public void go () {

        SETUP_DIR = System.getProperty(PROP_SETUP_DIR);
        try {
            itsDebugWriter = new PrintWriter(new FileWriter(SETUP_DIR 
                                                            + File.separator
                                                            + "install.log"));
            itsDebugWriter.println("Starting installation...");
            log("here we go...");
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_CREATING_INSTALL_LOG,
                                            e);
            e.printStackTrace();
        }

        if ( !initUI() ) return;

        pack();
        setBounds( new Rectangle(75,75,350,120) );
        setVisible(true);

        AlertDialog a = new AlertDialog(this, 
                                        AlertDialog.TITLE_WARNING,
                                        AlertDialog.MSG_PLEASE_BE_HONEST_WITH_NET_CONNECTION);
    }
    
    public boolean initUI () {
        
        // Init basic UI
        try {
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        } catch ( Exception e ) {
            log("Error initializing UI: " + e.toString());
            e.printStackTrace(itsDebugWriter);
            return false;
        }

        itsAudioFormatCombo = new JComboBox();
        itsAudioFormatVec = new Vector();

        itsBitrateCombo = new JComboBox();

        itsServerIP = new JTextField(SERVER_IP);
        itsServerPort = new JTextField(SERVER_PORT);
        
        itsInstallButton = new JButton(TEXT_INSTALL_BUTTON);
        itsInstallButton.addActionListener(this);

        itsCancelButton = new JButton(TEXT_CANCEL_BUTTON);
        itsCancelButton.addActionListener(this);

        JPanel aMainPanel = new JPanel();
        aMainPanel.setLayout(new GridLayout(4, 2));
        aMainPanel.add(new JLabel("Audio Format:"));
        aMainPanel.add(itsAudioFormatCombo);
        aMainPanel.add(new JLabel("Internet Connection:"));
        aMainPanel.add(itsBitrateCombo);
        aMainPanel.add(new JLabel("Server IP Address:"));
        aMainPanel.add(itsServerIP);
        aMainPanel.add(new JLabel("Server Port:"));
        aMainPanel.add(itsServerPort);

        JPanel aButtonPanel = new JPanel();
        aButtonPanel.setLayout(new BorderLayout());
        aButtonPanel.add(itsInstallButton, BorderLayout.WEST);
        aButtonPanel.add(itsCancelButton, BorderLayout.EAST);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(aMainPanel, BorderLayout.CENTER);
        getContentPane().add(aButtonPanel, BorderLayout.SOUTH);

        addWindowListener(this);

        populateAudioFormats();

        populateBitrates();

        return true;
    }

    private void populateAudioFormats() {
        int i, j, k;

        for ( i=0; i<FREQUENCIES.length; i++ ) {
            for ( j=0; j<SAMPLE_BITS.length; j++ ) {
                for ( k=0; k<NUM_CHANNELS.length; k++ ) {
                         checkAudioFormat( FREQUENCIES[i],
                                           FREQUENCY_NAMES[i], 
                                           SAMPLE_BITS[j],
                                           NUM_CHANNELS[k] );
                }
            }
        }
    }

    private void checkAudioFormat( float theFreq,
                                   String theFreqName,
                                   int theSampleBits, 
                                   int theNumChannels ) {

        AudioFormat anAudioFormat = new AudioFormat( theFreq,
                                                     theSampleBits,
                                                     theNumChannels,
                                                     ISBIGENDIAN,
                                                     PCM_SIGNED );

        DataLine.Info info = new DataLine.Info(TargetDataLine.class, 
                                               anAudioFormat);
        
        if (!AudioSystem.isLineSupported(info)) {
            return;
        }

        String aFormatDesc = theFreqName + ", " + theSampleBits + "-bit, ";
        aFormatDesc += (theNumChannels == 1) ? "mono" : "stereo";
       
        itsAudioFormatVec.add(anAudioFormat);
        itsAudioFormatCombo.addItem(aFormatDesc);
    }

    private void populateBitrates() {
        int i;
        for ( i=0; i<BITRATES.length; i++ ) {
            itsBitrateCombo.addItem(BITRATES_DESC[i]);
        }
    }

    public void actionPerformed(ActionEvent e) {
        if ( e.getSource() == itsCancelButton ) {
            System.exit(0);
        }
        if ( e.getSource() == itsInstallButton ) {
            if ( verifyFields() ) {
                doInstall();
            }
        }
    }

    private boolean verifyFields () {
        
        InetAddress aServerIP = null;
        
        try {
            aServerIP = InetAddress.getByName(itsServerIP.getText());
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_INVALID_IP_ADDRESS,
                                            e);
            e.printStackTrace(itsDebugWriter);
            return false;
        }

        try {
            int port = Integer.parseInt(itsServerPort.getText());
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_INVALID_PORT,
                                            e);
            e.printStackTrace(itsDebugWriter);
            return false;
        }

        // Make sure internet speed is fast enough
        try {
            int aBitrateToUse = BITRATES[itsBitrateCombo.getSelectedIndex()];
            if ( aBitrateToUse == -1 ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_INTERNET_CONNECTION_TOO_SLOW);
                return false;
            }
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_INVALID_PORT,
                                            e);
            e.printStackTrace(itsDebugWriter);
            return false;
        }        

        return true;
    }

    private void doInstall() {
        // Create the installation directory
        String anInstallDir = System.getProperty(PROP_INSTALL_DIR);
        File aDir = null;
        try {
            aDir = new File(anInstallDir);
            if ( !aDir.exists() && !aDir.mkdirs() ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_CREATING_INSTALL_DIR);
                return;
            }
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_CREATING_INSTALL_DIR,
                                            e);
            e.printStackTrace(itsDebugWriter);
            return;
        }

        // Copy the jar files and source zip to the installation directory
        try {
            copyFile(SETUP_DIR + File.separator + "opendjclient.jar", 
                     aDir.getAbsolutePath() + File.separator + "opendjclient.jar");
            copyFile(SETUP_DIR + File.separator + "jrev.jar", 
                     aDir.getAbsolutePath() + File.separator + "jrev.jar");
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_COPYING_FILES_TO_INSTALL_DIR,
                                            e);
            log(e);
            return;
        }

        // Create the startClient.bat file in the installation directory
        PrintWriter aWriter = null;
        try {
            File aStartup = new File(aDir.getAbsolutePath() + File.separator + START_COMMAND_NAME);
            aWriter = new PrintWriter(new FileWriter(aStartup));
            aWriter.println("set JAVA=" + System.getProperty(PROP_JAVA));
            aWriter.println("set CLIENT_CLASS=org.opendj.client.Main");
            aWriter.println("set CLASSPATH=" + System.getProperty(PROP_RT) + ";"
                        + aDir.getAbsolutePath() + File.separator + "jrev.jar;"
                        + aDir.getAbsolutePath() + File.separator + "opendjclient.jar");
            aWriter.println("set PROP_FILE=" + aDir.getAbsolutePath() + File.separator + CONFIG_FILE_NAME);
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("%JAVA% -classpath %CLASSPATH% -Djava.compiler=NONE "
                            + "-Dopendj.props=%PROP_FILE% %CLIENT_CLASS% "
                            + " 1> " + aDir.getAbsolutePath() + File.separator + "opendj.out.log" 
                            + " 2> " + aDir.getAbsolutePath() + File.separator + "opendj.err.log");

        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_CREATING_STARTUP_SCRIPT,
                                            e);
            log(e);
            return;

        } finally {
            try { aWriter.close(); } catch ( Exception e2 ) {}
        }

        // Create the opendj.properties file in the installation directory
        try {
            File aConfig = new File(aDir.getAbsolutePath() + File.separator + CONFIG_FILE_NAME);

            AudioFormat anAudioFormat = (AudioFormat) itsAudioFormatVec.get(itsAudioFormatCombo.getSelectedIndex());
            int aSampleSize = anAudioFormat.getSampleSizeInBits();
            int aNumChannels = anAudioFormat.getChannels();
            int aFrameSize = (aSampleSize / 8) * aNumChannels;
            int aFrameRate = (int) anAudioFormat.getSampleRate();
            int aBitrateToUse = BITRATES[itsBitrateCombo.getSelectedIndex()];

            aWriter = new PrintWriter(new FileWriter(aConfig));
            aWriter.println("# OPENdj server IP and port");
            aWriter.println("server.host=" + itsServerIP.getText());
            aWriter.println("server.port=" + itsServerPort.getText());
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# The encoder to use");
            aWriter.println("encoder.class=org.opendj.client.java.EncoderImpl");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# The JRev logfile");
            aWriter.println("jrev.logfile=./jrev.log");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# These properties determine the audio format used");
            aWriter.println("# to capture sound on your sound card.");
            aWriter.println("# The default values are for 16-bit, 44.1kHz, stereo sound");
            aWriter.println("# Only change these values if the defaults don't work and you");
            aWriter.println("# really know what you're doing.  If you do change these values,");
            aWriter.println("# you can find out if they work by using the ");
            aWriter.println("# org.opendj.client.java.RawSoundTest program.");
            aWriter.print("jrev.encoding=");
            if ( PCM_SIGNED ) aWriter.println("PCM_SIGNED");
            else aWriter.println("PCM_UNSIGNED");
            aWriter.println("jrev.samplerate=" + aFrameRate);
            aWriter.println("jrev.samplebits=" + aSampleSize);
            aWriter.println("jrev.channels=" + aNumChannels);
            aWriter.println("jrev.framesize=" + aFrameSize);
            aWriter.println("jrev.framerate=" + aFrameRate);
            aWriter.print("jrev.bigendian=");
            if ( ISBIGENDIAN ) aWriter.println("true");
            else aWriter.println("false");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# This determines the bitrate to send to the icecast server");
            aWriter.println("icecast.bitrate.use=" + aBitrateToUse);
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# This is the command that is used to encode the raw sound signal.");
            aWriter.println("# Some of the options you use here will be determined by the audio");
            aWriter.println("# capture options above.  The default values assume a big-endian,");
            aWriter.println("# 16-bit, 44.1kHz stereo source, and will result in an MP3 stream.");
            aWriter.println("# Only change this property if your really know what you are doing.  ");
            aWriter.println("# If you do change this value, you can find out if it works by using ");
            aWriter.println("# the org.opendj.client.java.LameTest program.");
            aWriter.print("jrev.lameCommand=");
            aWriter.print(Util.stringReplace(System.getProperty(PROP_LAME), "\\", "\\\\"));
            aWriter.print(" -r -h ");
            if ( aNumChannels == 2 ) aWriter.print("-m s");
            else aWriter.print("-m m");
            if ( ISBIGENDIAN == false && aSampleSize > 8 ) aWriter.print(" -x ");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# These are the various buffer sizes used in reading raw data from the");
            aWriter.println("# sound card and sending data to the LAME encoder -- you should ");
            aWriter.println("# probably not change these");
            aWriter.println("jrev.lineInBufferSize=4096");
            aWriter.println("jrev.lameBufferSize=4096");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.println("# This is the length of the test interval if you are testing things");
            aWriter.println("# using the RawSoundTest or LameTest programs (both are in the");
            aWriter.println("# org.opendj.client.java package).");
            aWriter.println("test.seconds=15");
            aWriter.println(Util.EMPTY_STRING);
            aWriter.close();

            // Copy config file to "opendj.properties.original"
            copyFile( aConfig.getAbsolutePath(), 
                      aConfig.getAbsolutePath() + ".original" );

        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_CREATING_CONFIG_FILE,
                                            e);
            log(e);
            return;

        } finally {
            try { aWriter.close(); } catch ( Exception e2 ) {}
        }

        try {
            // Create a user.home\Start Menu\Programs\OPENdj directory
            String aUserHome = System.getProperty("user.home");
            File aStartMenu = new File(aUserHome + File.separator 
                                       + "Start Menu" + File.separator
                                       + "Programs" + File.separator
                                       + "OPENdj");
            if ( !aStartMenu.exists() && !aStartMenu.mkdirs() ) {
                AlertDialog a = new AlertDialog(this, 
                                                AlertDialog.TITLE_ERROR,
                                                AlertDialog.MSG_ERROR_CREATING_START_MENU_SHORTCUT);
                return;
            }

            // Copy the START_COMMAND_NAME to this new directory
            createLink( anInstallDir + File.separator + START_COMMAND_NAME, aStartMenu.getAbsolutePath() + File.separator + "OPENdj.lnk" );

        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_CREATING_START_MENU_SHORTCUT,
                                            e);
            log(e);
            return;

        }

        AlertDialog a = new AlertDialog(this, 
                                        AlertDialog.TITLE_DONE,
                                        AlertDialog.MSG_INSTALL_COMPLETED);
        while ( !a.wasButtonPushed() ) {
            Util.sleep(250);
        }
        System.exit(0);
    }

    private void copyFile ( String theFileName, String theDest ) throws IOException, InterruptedException {
        log("copyFile('" + theFileName + "', '" + theDest + "') called...");

        FileInputStream fi = null;
        FileOutputStream fo = null;

        try {
            fi = new FileInputStream(theFileName);
            fo = new FileOutputStream(theDest);
            Installer.copyStream(fi, fo);
        } finally {
            try { fi.close(); } catch ( Exception e ) {}
            try { fo.close(); } catch ( Exception e ) {}
        }
    }

    private void createLink ( String theTarget, String theLink )  throws IOException, InterruptedException {
        log("createLink('" + theTarget + "', '" + theLink + "') called...");
        Process p = null;
        p = Runtime.getRuntime().exec(SETUP_DIR + File.separator + "createLink.exe \"" + theTarget + "\" \"" + theLink + "\"");
        p.waitFor();
    }

    public void finalize () {
        try { itsDebugWriter.close(); } catch ( Exception e ) {}
    }

    private void log ( String s ) {
        try {
            itsDebugWriter.println(s);
            itsDebugWriter.flush();
        } catch ( Exception e ) {
            AlertDialog a = new AlertDialog(this, 
                                            AlertDialog.TITLE_ERROR,
                                            AlertDialog.MSG_ERROR_WRITING_INSTALL_LOG,
                                            e);
            e.printStackTrace();
        }
    }
    public void log ( Throwable t ) {
        log(Installer.getStackTrace(t));
    }
    
    public static String getStackTrace( Throwable theT ) {
        if ( theT == null ) return "THROWABLE-WAS-NULL";
        try {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            theT.printStackTrace(pw);
            String rval = sw.toString();
            pw.close();
            return rval;
        } catch ( Exception e ) {
            return "\n\ngetStackTrace GENERATED EXCEPTION: '" 
                + e.toString() + "'\n\n";
        }
    }
    public static void copyStream( InputStream theInStream, OutputStream theOutStream ) throws IOException {
        
        int aNumBytesRead = 0;
        byte[] aBuffer = new byte[DEFAULT_BUFFER_SIZE];
        while ( aNumBytesRead >= 0 ) { 
            aNumBytesRead = theInStream.read(aBuffer, 0, aBuffer.length);
            if ( aNumBytesRead > 0 ) {
                theOutStream.write(aBuffer, 0, aNumBytesRead);
            }
        }
    }
    public static final int DEFAULT_BUFFER_SIZE = 4096;

    public void windowClosing(WindowEvent e) {
        System.exit(0);
    }
    public void windowOpened(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
}
