/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-client/src/org/opendj/client/unix/EncoderConfig.java,v 1.1.1.1 2001/09/29 20:22:00 jonathan Exp $
 * $Log: EncoderConfig.java,v $
 * Revision 1.1.1.1  2001/09/29 20:22:00  jonathan
 * Imported opendj-client
 *
 */
package org.opendj.client.unix;

import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Properties;
import org.opendj.client.Main;
import org.opendj.share.Util;
import org.opendj.share.ChannelTime;
import org.opendj.share.DjBean;

public class EncoderConfig {

    public static final String PROP_CONFIG_TEMPLATE = "liveice.config.template";
    public static final String PROP_CONFIG          = "liveice.config";

    public static final String SYMBOL_SERVER   = "$$SERVER$$";
    public static final String SYMBOL_PORT     = "$$PORT$$";
    public static final String SYMBOL_PASSWORD = "$$PASSWORD$$";
    public static final String SYMBOL_MOUNT    = "$$MOUNT$$";
    public static final String SYMBOL_SHOWNAME = "$$SHOWNAME$$";
    public static final String SYMBOL_GENRE    = "$$GENRE$$";
    public static final String SYMBOL_URL      = "$$URL$$";
    public static final String SYMBOL_BITRATE  = "$$BITRATE$$";

    public static void generateConfig( Properties theProps,
                                       DjBean theDj,
                                       ChannelTime theChannelTime,
                                       InetAddress theRemoteAddr ) throws IOException {
        FileInputStream fi = null;
        FileOutputStream fo = null;
        ByteArrayInputStream bytes = null;
        String aConfigContents = null;

        try {
            fi = new FileInputStream(theProps.getProperty(PROP_CONFIG_TEMPLATE));
            aConfigContents = Util.readStreamAsString(fi);
            fi.close();

            aConfigContents
                = Util.stringReplace(aConfigContents, 
                                     SYMBOL_SERVER, 
                                     theProps.getProperty(Main.PROP_ICECAST_HOST));

            // Just do this to make sure the port is a number
            // It will throw a RuntimeException (NumberFormatException)
            // if the port is not a number.  Note that this will succeed
            // if the port is a negative number.
            String aPort = theProps.getProperty(Main.PROP_ICECAST_PORT);
            int aPortInt = Integer.parseInt(aPort);
            aConfigContents = Util.stringReplace(aConfigContents, 
                                                 SYMBOL_PORT, 
                                                 aPort);
            aConfigContents = Util.stringReplace(aConfigContents,
                                                 SYMBOL_PASSWORD,
                                                 Util.nz(theProps.getProperty(Main.PROP_ICECAST_PASSWORD)));
            aConfigContents = Util.stringReplace(aConfigContents,
                                                 SYMBOL_MOUNT,
                                                 Util.stripUpTo(theChannelTime.path, "/"));
            aConfigContents = Util.stringReplace(aConfigContents, 
                                                 SYMBOL_SHOWNAME, 
                                                 theDj.getshowname());
            aConfigContents = Util.stringReplace(aConfigContents, 
                                                 SYMBOL_GENRE, 
                                                 theDj.getgenreval());
            String aWebURL = theDj.getwebpage();
            if ( aWebURL != null && aWebURL.length() > 0 ) {
                aConfigContents = Util.stringReplace(aConfigContents, 
                                                     SYMBOL_URL, 
                                                     aWebURL);
            }
            aConfigContents = Util.stringReplace(aConfigContents, 
                                                 SYMBOL_BITRATE, 
                                                 theProps.getProperty(Main.PROP_USE_BITRATE));

            fo = new FileOutputStream(theProps.getProperty(PROP_CONFIG));
            bytes = new ByteArrayInputStream(aConfigContents.getBytes());
            Util.copyStream(bytes, fo);
            fo.close();

        } finally {
            try { fi.close(); } catch ( Exception e ) {}
            try { fo.close(); } catch ( Exception e ) {}
        }
    }

    private EncoderConfig() {}
}
